@@ -13,7 +13,7 @@ from masci_tools.io.common_functions import get_alat_from_bravais
 __copyright__ = (u'Copyright (c), 2023, Forschungszentrum Jülich GmbH, '
                  'IAS-1/PGI-1, Germany. All rights reserved.')
 __license__ = 'MIT license, see LICENSE.txt file'
-__version__ = '0.1.2'
+__version__ = '0.1.4'
 __contributors__ = (u'Philipp Rüßmann', u'Raffaele Aliberti')
 
 ##############################################################################
@@ -25,7 +25,7 @@ def convert_to_imp_cls(host_structure, imp_info):
     convert imp info to rcls form
     """
     if 'imp_cls' in imp_info.get_dict():
-        clust1 = imp_info['imp_cls']
+        clust1 = np.array(imp_info['imp_cls'])
         imp_info_cls = imp_info
     else:
         # convert Zimp, Rcut info to imp_cls info
@@ -56,7 +56,7 @@ def get_imp_cls_add(host_structure, add_position):
     ilayer = add_position['ilayer']
     imp_info2 = orm.Dict({'ilayer_center': ilayer, 'Zimp': [Zadd], 'Rcut': 1e-5})
     # old version is too slow:
-    # clust2 = get_scoef_single_imp(host_structure, imp_info2)
+    #clust2 = get_scoef_single_imp(host_structure, imp_info2)
     # new version creates the array without calling the get_scoef_single_imp function:
     clust2 = np.array([[0., 0., 0., ilayer + 1, 0., 0.]])
     return imp_info2, clust2
@@ -80,9 +80,9 @@ def get_r_offset(clust1, clust2, host_structure, add_position):
     alat = get_alat_from_bravais(np.array(host_structure.cell), host_structure.pbc[2])
     r_out_of_plane /= alat
 
-    # remove spurious offsets that might be there due to the choice of the unit cell positions
-    r_out_of_plane = np.round(r_out_of_plane, 7)
-    r_out_of_plane[:2] %= 1  # modulo 1 for x and y coordinate
+    # # remove spurious offsets that might be there due to the choice of the unit cell positions
+    # r_out_of_plane = np.round(r_out_of_plane, 7)
+    # r_out_of_plane[:2] %= 1  # modulo 1 for x and y coordinate
 
     # calculate in-plane vector from da, db inputs
     da = add_position.get_dict().get('da', 0)
@@ -132,10 +132,16 @@ def get_imp_info_add_position(add_position, host_structure, imp_info):
     # combine cluster information
     pos_exists_in_imp1, _ = pos_exists_already(clust1, clust2)
     if pos_exists_in_imp1:
-        raise ValueError('Additional position exists already in impurity cluster.')
+        # If the position exists already we simply skip the addition of the new scanning position
+        return None
+        #raise ValueError('Additional position exists already in impurity cluster.')
     cluster_combined, rimp_rel_combined, _, _ = combine_clusters(clust1, clust2_offset, False, debug=False)
     # combine the zimp arrays
-    zimp_combined = imp_info['Zimp'] + imp_info2['Zimp']
+    zimp1 = imp_info['Zimp']
+    if not isinstance(zimp1, list):
+        # convert to list if necessary
+        zimp1 = [zimp1]
+    zimp_combined = zimp1 + imp_info2['Zimp']
     # now combine the imp info node
     imp_info_combined = orm.Dict({'imp_cls': cluster_combined, 'Zimp': zimp_combined, 'Rimp_rel': rimp_rel_combined})
 
@@ -247,78 +253,76 @@ def create_combined_potential_node_cf(add_position, host_calc, imp_potential_nod
 
 
 def STM_pathfinder(host_remote):
-    """This function is used to help visualize the scanned positions
-       and the symmetries that are present in the system
+    """
+    Calcfunction that gives back the structural information of the film, and the symmetries of the system
 
-    inputs::
-    host_remote : RemoteData : The Remote data contains all the information needed to create the path to scan
+    inputs ::
 
-    outputs::
-    struc_info : Dict  : Dictionary containing the structural information of the film
-    matrices   : Array : Array containing the matrices that generate the symmetries of the system
+           host_remote : Remote_data : node containing the remote data of the host material
+
+    return ::
+
+           plane_vectors   : list : list containing the 2 in plane vectors that span the surface.
+           unique_matrices : list : list of matrices, contains the 2x2 matrices that constitue the symmetry operations of the system.
     """
 
-    def info_creation(structure):
-        from ase.spacegroup import get_spacegroup
-        # List of the Bravais vectors
-        vec_list = structure.cell.tolist()
-
-        # Find the Bravais vectors that are in plane vectors
-        plane_vectors = {'plane_vectors': [], 'space_group': ''}
-        for vec in vec_list:
-            # Is this sufficient to find all the in-plane vectors?
-            if vec[2] == 0:
-                plane_vectors['plane_vectors'].append(vec[:2])
-
-        space_symmetry = get_spacegroup(structure)
-        plane_vectors['space_group'] = space_symmetry.no
-
-        return plane_vectors
-
-    def symmetry_finder(struc_info):
-        from ase.spacegroup import Spacegroup
-        # Here we get the symmetry operations that are possible
-        symmetry_matrices = Spacegroup(struc_info['space_group'])
-
-        # Reduce the dimensionality, we only want the 2D matrices
-        matrices = []
-        for element in symmetry_matrices.get_rotations():
-            matrices.append(element[:2, :2])
-
-        # Uniqueness of the elements must be preserved
-        unique_matrices = []
-        for matrix in matrices:
-            if not any(np.array_equal(matrix, m) for m in unique_matrices):
-                unique_matrices.append(matrix)
-
-        return unique_matrices
+    from pymatgen.symmetry.analyzer import SpacegroupAnalyzer, SymmOp
 
     struc = find_parent_structure(host_remote)
     # clone the structure since it has already been saved in AiiDA and cannot be modified
     supp_struc = struc.clone()
 
     # If the structure is not periodic in every direction we force it to be.
-    if not supp_struc.pbc[2]:
-        # find film thickness
-        zs = np.array([i.position[2] for i in supp_struc.sites])
-        z = zs.max() - zs.min() + 5  # add 5 to have a unit cell larger than the considered film thickness
-        # set third bravais vector along z direction
-        cell = supp_struc.cell
-        cell[2] = [0, 0, z]
-        supp_struc.set_cell(cell)
-        # change periodic boundary conditions to periodic
-        supp_struc.pbc = (True, True, True)
+    supp_struc.pbc = (True, True, True)
 
-    # ASE struc
-    ase_struc = supp_struc.get_ase()
+    # Pymatgen struc
+    py_struc = supp_struc.get_pymatgen()
 
-    # Structural informations are stored here
-    struc_info = info_creation(ase_struc)
+    struc_dict = py_struc.as_dict()
+    # Find the Bravais vectors that are in-plane vectors (assumes 2D structure)
+    plane_vectors = {'plane_vectors': [], 'space_group': ''}
+    for vec in struc_dict['lattice']['matrix']:
+        # Is this sufficient to find all the in-plane vectors?
+        if vec[2] == 0 or (struc.pbc[2] and (vec[0] + vec[1]) > 0):
+            plane_vectors['plane_vectors'].append(vec[:2])
+    # finally check if setting of plane_vectors worked
+    if 'plane_vectors' not in plane_vectors:
+        raise ValueError('Could not set "plane_vectors" in STM_pathfinder')
 
-    # The structural informations are then used to find the symmetries of the system
-    symm_matrices = symmetry_finder(struc_info)
+    # Here we get the symmetry operations that are possible
+    symmetry_matrices = SpacegroupAnalyzer(py_struc).get_point_group_operations(cartesian=True)
 
-    return struc_info, symm_matrices
+    plane_vectors['space_group'] = SpacegroupAnalyzer(py_struc).get_symmetry_dataset()['number']
+
+    # Here we get the symmetry rotations
+
+    supp_mat = []
+
+    # Get the affine representation of the matrices
+    for symmops in range(len(symmetry_matrices)):
+        supp_mat.append(np.array(SymmOp.as_dict(symmetry_matrices[symmops])['matrix'][:3]))
+
+    # Get only the rotation matrices and correct the numerical error
+    rot_mat = []
+
+    # Take only the matrices for the in-plane rotations
+    for elements in range(len(supp_mat)):
+        rot_mat.append(supp_mat[elements][0:2, 0:2])
+
+    # Sometimes it will happen that some rotation matrices projected to the 2D space will have the same representation.
+    # here we only take the unique ones
+    unique_matrices = []
+    for matrix in rot_mat:
+        if not any(np.array_equal(matrix, m) for m in unique_matrices):
+            unique_matrices.append(matrix)
+
+    # Round off the numerical error
+    for elements in range(len(unique_matrices)):
+        for rows in range(len(unique_matrices[elements])):
+            for cols in range(len(unique_matrices[elements][rows])):
+                unique_matrices[elements][rows][cols] = round(unique_matrices[elements][rows][cols])
+
+    return plane_vectors, unique_matrices
 
 
 @engine.calcfunction
@@ -336,53 +340,65 @@ def STM_pathfinder_cf(host_structure):
 # lattice generation (function of lattice plot)
 
 
-def lattice_generation(x_len, y_len, rot, vec):
-    import math
-    """
-    x_len : int  : value to create points between - x and x.
-    y_len : int  : value to create points between - y and y.
-    rot   : list : list of the rotation matrices given by the symmetry of the system.
-    vec   : list : list containing the two Bravais vectors.
+def lattice_generation(rot, vec, x_start, y_start, xmax, ymax):
     """
 
-    # Here we create a grid in made of points whic are the linear combination of the lattice vectors
-    lattice_points = []
+        inputs ::
 
-    for i in range(-x_len, x_len + 1):
-        lattice_points_col = []
-        for j in range(-y_len, y_len + 1):
-            p = [i * x + j * y for x, y in zip(vec[0], vec[1])]
-            lattice_points_col.append(p)
-        lattice_points.append(lattice_points_col)
+        x_len  : int  : value to create points between - x and x.
+        y_len  : int  : value to create points between - y and y.
+        rot    : list : list of the rotation matrices given by the symmetry of the system.
+        vec    : list : list containing the two Bravais vectors.
+        start_x: int  : starting value for the lattice generation in the x direction
+        start_y: int  : starting value for the lattice generation in the y direction
 
-    # Eliminiatio of the symmetrical sites
-    points_to_eliminate = []
+        return ::
 
-    for i in range(-x_len, x_len + 1):
-        for j in range(-y_len, y_len + 1):
-            if ((lattice_points[i][j][0] > 0 or math.isclose(lattice_points[i][j][0], 0, abs_tol=1e-3)) and
-                (lattice_points[i][j][1] > 0 or math.isclose(lattice_points[i][j][1], 0, abs_tol=1e-3))):
-                for element in rot[1:]:
-                    point = np.dot(element, lattice_points[i][j])
-                    if point[0] >= 0 and point[1] >= 0:
-                        continue
-                    else:
-                        points_to_eliminate.append(point)
+        points_to_eliminate : list : list of list containing the (x,y) positions to NOT to
+                                     be scanned
+        points_to_scan      : list : list of list containing the (x,y) positions to BE be
+                                     scanned
+        """
+
+    # Here we create a grid  made of points which are the linear combination of the lattice vectors
+    x_len = xmax * 10
+    y_len = ymax * 10  # maybe there is a way to make this more efficient...
+
+    x_interval = [i for i in range(-x_len, x_len)]
+
+    y_interval = [i for i in range(-y_len, y_len)]
 
     points_to_scan = []
 
-    for i in range(-x_len, x_len + 1):
-        for j in range(-y_len, y_len + 1):
-            eliminate = False
-            for elem in points_to_eliminate:
-                # Since there can be some numerical error in the dot product we use the isclose function
-                if (
-                    math.isclose(elem[0], lattice_points[i][j][0], abs_tol=1e-4) and
-                    math.isclose(elem[1], lattice_points[i][j][1], abs_tol=1e-4)
-                ):
-                    eliminate = True
-            if not eliminate:
-                points_to_scan.append(lattice_points[i][j])
+    # Now we only generate points in the first quadrant and the we use the symmetry analysis
+    # To visualize the other unscanned sites
+
+    for i in x_interval:
+        points_to_scan_col = []
+        for j in y_interval:
+            p = [i * x + j * y for x, y in zip(vec[0], vec[1])]
+            if ((p[0] < 0 or p[0] > xmax) or (p[1] < 0 or p[1] > ymax)) or (p[0] < x_start or p[1] < y_start):
+                continue
+            else:
+                points_to_scan_col.append(p)
+        if len(points_to_scan_col) != 0:
+            points_to_scan.append(points_to_scan_col)
+
+    #print(lattice_points)
+    points_to_eliminate = []
+
+    for i in range(len(points_to_scan)):
+        for j in range(len(points_to_scan[i])):
+            #print(lattice_points[i][j])
+            #if lattice_points[i][j][0] >= 0 and lattice_points[i][j][1] >= 0:
+            for element in rot[1:]:
+                point = np.dot(element, points_to_scan[i][j])
+                #if point[0] >= 0 and point[1] >=0:
+                #    continue
+                #else:
+                points_to_eliminate.append(point)
+
+    #print(point_to_eliminate)
 
     return points_to_eliminate, points_to_scan
 
@@ -392,13 +408,31 @@ def lattice_generation(x_len, y_len, rot, vec):
 
 
 def lattice_plot(plane_vectors, symm_vec, symm_matrices, grid_length_x, grid_length_y):
+    """
+        Helper tool to plot the position that will be scanned in the submission of the
+        kkr_STM_wc workchain
+
+        inputs ::
+
+        plane_vectors : list : list containing the Bravais vector of the 2D lattice
+        symm_vec      : bool : Toggle to show or not the Bravais vectors in the plot
+        symm_matrices : list : list of the point-group symmetries matrices of the system
+        grid_length_x : int  : scanning distance in the x direction
+        grid_length_y : int  : scanning distance in the y direction
+
+        return ::
+
+        None
+
+    """
+
     #from aiida_kkr.tools.tools_STM_scan import lattice_generation
     import matplotlib.pyplot as plt
     from matplotlib.lines import Line2D
 
     origin = np.array([[0, 0], [0, 0]])
     # Generation of the points to plot
-    unused, used = lattice_generation(grid_length_x, grid_length_y, symm_matrices, plane_vectors)
+    unused, used = lattice_generation(symm_matrices, plane_vectors, 0, 0, grid_length_x, grid_length_y)
 
     # Plotting of the points
     for element in unused:
@@ -446,32 +480,34 @@ def lattice_plot(plane_vectors, symm_vec, symm_matrices, grid_length_x, grid_len
 def find_linear_combination_coefficients(plane_vectors, vectors):
     from operator import itemgetter
     """This helper function takes the planar vectors and a list of vectors
-       and return the coefficients in the base of the planar vectors"""
+       and return the coefficients in the base of the planar vectors
+
+       inputs ::
+
+       plane vectors: list : list of list of the form [[a_x, a_y], [b_x, b_y]]
+
+       returns ::
+
+       indices : list : list of list of the form [[int_1, int_1]...[int_n, int_n]]
+                        the integers refers to how many times that specific vectors is
+                        present in the linear combination r = int_x * a + int_2 * b
+
+       """
 
     # Formulate the system of equations Ax = b
     A = np.vstack((plane_vectors[0], plane_vectors[1])).T
-
-    # Solve the system of equations using least squares method
-    data = []
-    for element in vectors:
-        b = element
-        # We use the least square mean error procedure to evaulate the units of da and db
-        # lstsq returns: coeff, residue, rank, and singular value
-        # We only need the coefficient.
-        data.append(np.linalg.lstsq(A, b, rcond=None))
-
+    # inverse of A matrix
+    Ainv = np.matrix(A)**(-1)
     indices = []
-    for element in data:
-        supp = []
-        for elem in element[0]:
-            # Here we round to an integer, this is because of the numerical error
-            # which is present inside the calculation.
-            supp.append(round(elem))
-        indices.append(supp)
+    # loop over flattened list
+    for vec in np.array(vectors).reshape(-1, 2):
+        # get indices from x = A^{-1}.b
+        index = np.array((Ainv * np.matrix(vec).transpose()).transpose()).reshape(2)
+        indices.append(index)
+    # make sure to have integer values
+    indices = np.round(np.array(indices), 0)
 
-    # Before returning the indices, we reorder them first from the lowest to the highest valued
-    # on the x axis and then from the lowest to the highest on the y axis.
-
-    indices = sorted(indices, key=itemgetter(0, 1))
+    isort = indices[:, 0] + indices[:, 1] / (10 * max(indices[:, 0]))
+    indices = indices[isort.argsort()]
 
     return indices
